#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "parse_fat.h"

int main(int argc, char *argv[])
{
struct boot_sector_t boot_sector;
struct fat_info_t fat_info;
FILE *in;
unsigned long partition_offset=0;
unsigned char sector[512];
int i,t;

  printf("dump_fat - Copyright 2007 Michael Kohn\n");
  printf("Version: November 21, 2007\n");
  printf("Webpage: http://www.mikekohn.net/\n");
  printf("  Email: mike@mikekohn.net\n\n");

  if (argc!=2)
  {
    printf("Usage: fat_parse infile.img\n");
    exit(0);
  }

  in=fopen(argv[1],"rb");
  if (in==0)
  {
    printf("Couldn't open input file %s\n",argv[1]);
    exit(1);
  }

  fread(sector,1,512,in);

  if (sector[0x1fe]==0x55 && sector[0x1ff]==0xaa && sector[0x1bc]==0 && sector[0x1bd]==0)
  {
    printf("Detected MBR\n\nPartitions:\n");

    for (t=(0x1be); t<((0x1be)+(16*4)); t=t+16)
    {
      if (sector[t+4]==0) break;
      printf("Bootable: ");

      if (sector[t]==0x80) printf("Yes\n");
        else
      if (sector[t]==0x00) printf("No\n");
        else
      {
        if (t==0x1be) break;
        printf("Malformed\n");
      }

      printf("     Partition Type: %d ",sector[t+4]);
      if (sector[t+4]==0x06) printf("(FAT16)");
        else
      if (sector[t+4]==0x82) printf("(Linux Swap)");
        else
      if (sector[t+4]==0x83) printf("(Linux)");
      printf("\n");

      printf(" Address 1st Sector: c=%d h=%d s=%d\n",sector[t+1],sector[t+2],sector[t+3]);
      printf("Address Last Sector: c=%d h=%d s=%d\n",sector[t+5],sector[t+6],sector[t+7]);
      i=sector[t+12]+(sector[t+13]<<8)+(sector[t+12]<<14)+(sector[t+15]<<24);
      printf("Length Of Partition: %d sectors (%d bytes)\n",i,i*512);

      partition_offset=sector[t+8]|(sector[t+9]<<8)|(sector[t+10]<<16)|(sector[t+11]<<24);
      printf("    Logical Address: sector %ld, byte %ld\n",partition_offset,partition_offset*512);
      partition_offset*=512;
      printf("\n");
    }
  }

  fseek(in,partition_offset,SEEK_SET);

  if (parse_boot_sector(in,&boot_sector)!=0)
  {
    printf("Not known FAT file type\n");
  }
    else
  {
    print_boot_sector(&boot_sector);

    if (strcmp(boot_sector.file_sys_type,"FAT12   ")==0)
    { fat_info.type=12; }
      else
    if (strcmp(boot_sector.file_sys_type,"FAT16   ")==0)
    { fat_info.type=16; }
      else
    if (strcmp(boot_sector.file_sys_type,"FAT32   ")==0)
    { fat_info.type=32; }
      else
    { fat_info.type=8; }

    fat_info.offset_to_fat=boot_sector.reserved_sector_count*
                           boot_sector.bytes_per_sector;

    fat_info.offset_to_data=fat_info.offset_to_fat+
                            (boot_sector.num_alloc_tables*
                             boot_sector.sectors_per_table*
                             boot_sector.bytes_per_sector);

#ifdef DEBUG
printf("Offset to FAT: %ld\n",fat_info.offset_to_fat);
printf("Offset to DAT: %ld\n",fat_info.offset_to_data);
printf("         Type: %d\n\n",fat_info.type);
#endif
    parse_dir(in,fat_info.offset_to_data,partition_offset);
  }

  fclose(in);

  return 0;
}


