#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "parse_fat.h"

int main(int argc, char *argv[])
{
struct boot_sector_t boot_sector;
struct fat_info_t fat_info;
char filename[1024];
FILE *in,*out=0;
unsigned long partition_offset=0;
unsigned char sector[512];
int jpeg_count=0;
int i,t;
int frame_len=0,jpeg;
int frame_flag=0,ptr;

  printf("recover_fat_jpegs - Copyright 2007-2008 Michael Kohn\n");
  printf("Version: April 4, 2008\n");
  printf("Webpage: http://www.mikekohn.net/\n");
  printf("  Email: mike@mikekohn.net\n\n");

  if (argc!=2)
  {
    printf("Usage: recover_fat_jpegs infile.img\n");
    exit(0);
  }

  in=fopen(argv[1],"rb");
  if (in==0)
  {
    printf("Couldn't open input file %s\n",argv[1]);
    exit(1);
  }

  fread(sector,1,512,in);

  if (sector[0x1fe]==0x55 && sector[0x1ff]==0xaa && sector[0x1bc]==0 && sector[0x1bd]==0)
  {
    printf("Detected MBR\n\nPartitions:\n");

    for (t=(0x1be); t<((0x1be)+(16*4)); t=t+16)
    {
      if (sector[t+4]==0) break;
      printf("Bootable: ");

      if (sector[t]==0x80) printf("Yes\n");
        else
      if (sector[t]==0x00) printf("No\n");
        else
      {
        if (t==0x1be) break;
        printf("Malformed\n");
      }

      printf("     Partition Type: %d ",sector[t+4]);
      if (sector[t+4]==0x06) printf("(FAT16)");
        else
      if (sector[t+4]==0x82) printf("(Linux Swap)");
        else
      if (sector[t+4]==0x83) printf("(Linux)");
      printf("\n");

      printf(" Address 1st Sector: c=%d h=%d s=%d\n",sector[t+1],sector[t+2],sector[t+3]);
      printf("Address Last Sector: c=%d h=%d s=%d\n",sector[t+5],sector[t+6],sector[t+7]);
      i=sector[t+12]+(sector[t+13]<<8)+(sector[t+12]<<14)+(sector[t+15]<<24);
      printf("Length Of Partition: %d sectors (%d bytes)\n",i,i*512);

      partition_offset=sector[t+8]|(sector[t+9]<<8)|(sector[t+10]<<16)|(sector[t+11]<<24);
      printf("    Logical Address: sector %ld, byte %ld\n",partition_offset,partition_offset*512);
      partition_offset*=512;
      printf("\n");
    }
  }

  fseek(in,partition_offset,SEEK_SET);

  if (parse_boot_sector(in,&boot_sector)!=0)
  {
    printf("Not known FAT file type\n");
  }
    else
  {
    print_boot_sector(&boot_sector);

    if (strcmp(boot_sector.file_sys_type,"FAT12   ")==0)
    { fat_info.type=12; }
      else
    if (strcmp(boot_sector.file_sys_type,"FAT16   ")==0)
    { fat_info.type=16; }
      else
    if (strcmp(boot_sector.file_sys_type,"FAT32   ")==0)
    { fat_info.type=32; }
      else
    { fat_info.type=8; }

    fat_info.offset_to_fat=boot_sector.reserved_sector_count*
                           boot_sector.bytes_per_sector;

    fat_info.offset_to_data=fat_info.offset_to_fat+
                            (boot_sector.num_alloc_tables*
                             boot_sector.sectors_per_table*
                             boot_sector.bytes_per_sector);

#ifdef DEBUG
printf("Offset to FAT: %ld\n",fat_info.offset_to_fat);
printf("Offset to DAT: %ld\n",fat_info.offset_to_data);
printf("         Type: %d\n\n",fat_info.type);
#endif
    parse_dir(in,fat_info.offset_to_data,partition_offset);
  }

  fseek(in,fat_info.offset_to_data,SEEK_SET);
  jpeg=0;

  while(1)
  {
    if (fread(sector,1,512,in)==0) break;
    if (jpeg==0 && sector[0]==0xff && sector[1]==0xd8)
    {
      printf("Start of jpeg here %ld\n",ftell(in));
      sprintf(filename,"image%d.jpeg",++jpeg_count);
      out=fopen(filename,"wb");
      jpeg=1;
      frame_flag=0;
      frame_len=0;
    }

    if (jpeg==1)
    {
      for(ptr=0; ptr<512; ptr++)
      {
        putc(sector[ptr],out);

        if (frame_flag==0 && frame_len>0)
        {
          frame_len--;
          continue;
        }

        if (sector[ptr]==0xff && frame_flag==0) frame_flag=1;
          else
        if (frame_flag==1)
        {
          if (frame_len==-1)
          {
            if (sector[ptr]==0xd9)
            {
              fclose(out);
              jpeg=0;
              break;
            }
            frame_flag=0;
            continue;
          }

          if (sector[ptr]==0xda)
          {
            frame_len=-1;
            frame_flag=0;
          }
            else
          if (sector[ptr]!=0xd8)
          {
            // printf("frame_type=%02x\n",sector[ptr]);
            frame_flag++;
          }
            else
          {
            frame_flag=0;
            frame_len=-1;
          }
        }
          else
        if (frame_flag==2)
        {
          frame_len=((int)sector[ptr])<<8;
          frame_flag++;
        }
          else
        if (frame_flag==3)
        {
          frame_len|=sector[ptr];
          frame_len-=2;
          frame_flag=0;
          // printf("frame_len=%d\n",frame_len);
        }
      }
    }
  }

  printf("Found %d jpegs\n",jpeg_count);

  fclose(in);

  return 0;
}


